@@ -17,6 +17,53 @@ class ActionFailed(Exception):
     def __init__(self, action):
         self.message = f"'{action}' failed!'"
 
+class Attachment:
+    """Message attachment"""
+    def from_json(json):
+        if json is None: return None
+        return Attachment(json["filename"], json["url"], json["size"])
+
+    def __init__(self, filename, url, size):
+        if not isinstance(size, int): raise AttributeError(f"{type(size)} is not int type")
+        self.url = url
+        self.filename = filename,
+        self.size = size
+
+    def size(slef):
+        return self.size
+
+    def download(self):
+        import requests
+        return requests.get(self.url)
+
+    def save(self, path):
+        if not os.path.exists(path): return ""
+        file = self.download()
+        with open(os.path.join(path, filename), "wb") as f:
+            f.write(file.content)
+        return os.path.join(path, filename)
+    
+    def to_json(self):
+        return {"filename":self.filename, "size":self.size, "url":self.url}
+
+class Message:
+    """Message object used by the api"""
+    def from_json(json):
+        return Message(json["sender"], json["content"], json["channel"], Attachment.from_json(json["attachment"]), json["called"])
+
+    def __init__(self, sender, content, channel, attachment, called):
+        self.sender = sender
+        self.content = content
+        self.channel = channel
+        self.attachment = attachment
+        self.called = called
+
+    def add_called(self, called):
+        self.called = called
+
+    def to_json(self):
+        return {"sender":self.sender, "content":self.content, "channel":self.channel, "called":self.called, "attachment":self.attachment.to_json() if self.attachment is not None else None}
+
 def blockPrint():
     sys.stdout = open(os.devnull, 'w')
 
@@ -204,15 +251,13 @@ class API:
                         msg = self._retrive()
                         if self.sending:
                             self.buffer.append(msg)
-                        elif msg in self.call_list:
-                            retrived_call.append(msg)
-                        elif msg is None and retrived_call != []:
-                            call = threading.Thread(target=self.call_list[retrived_call[0]], args=[*retrived_call[1:], ])
-                            call.name = self.call_list[retrived_call[0]]
-                            retrived_call = []
-                            call.start()
                         elif retrived_call != []:
-                            retrived_call.append(msg)
+                            message = Message.from_json(msg)
+                            if message.called is not None and message.called in self.call_list:
+                                call = threading.Thread(target=self.call_list[message.called], args=[message, ])
+                                call.name = self.call_list[retrived_call[0]]
+                                retrived_call = []
+                                call.start()
                         elif msg is None:
                             self.connection_alive = False
                             self.socket.close()
@@ -241,16 +286,13 @@ class API:
         self._send({"Command":"Disconnect", "Value": reason})
         self.socket.close()
 
-    def create_function(self, name, help_text, callback, return_value=[NOTHING]):
+    def create_function(self, name, help_text, callback):
         """Creates a function in the connected bot.
         Return order: ChannelID, UserID, UserInput. The returned value depends on the return value, but the order is the same.
         """
         if self.valid:
-            if isinstance(return_value, (tuple, list)):
-                self.created_function_list.append([name, help_text, callback, sum(return_value)])
-                return_value = sum(return_value)
             self.sending = True
-            self._send({"Command":"Create", "Value": [name, help_text, name, return_value]})
+            self._send({"Command":"Create", "Value": [name, help_text, name]})
             while self.buffer == []:
                 sleep(0.1)
             tmp = self.buffer[0]
